/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on : 03.05.2003
 * Created by : s3595
 */
package at.spardat.xma.boot.exc;

import java.text.MessageFormat;
import java.util.Locale;

/**
 * @author s3595 <a href="mailto:schaeferc@spardat.at">Chris Schfer (CGS)</a>
 * @version 0.1.0
 */
public class BootRuntimeException extends Exception {

    /**
     * The code of this notification. Defaults to 0 if not set.
     */
    private int                 code_;

    /**
     * The message of this notification.
     */
    private String              message_;

    /**
     * The detail exception
     */
    private Throwable           detail_;

    /**
     * As for runtime exceptions:
     * Indicates if this exception may be shown to the end user or not. This is just
     * a hint for the presentation layer to decide if it should wrap this in
     * another exception.
     */
    protected boolean           showToEndUser_ = true;

    /**
     * For internal use only.
     */
    protected BootRuntimeException () {
    }

    /**
     * Constructs and sets the <tt>message</tt> from a format string as
     * defined in <tt>java.text.MessageFormat</tt>. The required parameters
     * must be contained in the <tt>params</tt> array.
     *
     * @param detail     detail exception.
     * @param message    string that either is a <tt>java.text.MessageFormat</tt>
     *                    or not, depending on <tt>params</tt>.
     * @param l          a <tt>java.util.Locale</tt> to format locale
     *                    dependent data types or <tt>null</tt> if the
     *                    <tt>params</tt> do not contain local specifics.
     * @param params the parameters of the message. If not <tt>null</tt>, <tt>message</tt>
     *                is expected to be a string compliant to <tt>MessageFormat</tt>.
     */
    public BootRuntimeException (Throwable detail, String message, Locale l, Object[] params) {
        // set detail
        if (detail != null) setDetail (detail);
        this.setMessage(message, l, params);
    }
    public BootRuntimeException (Throwable detail, String message) {
        // set detail
        if (detail != null) setDetail (detail);
        this.setMessage(message, null, null);
    }

    /**
     * Constructs and sets the <tt>message</tt> of this from a two
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     *
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 first message parameter
     * @param param2 second message parameter
     */
    public BootRuntimeException (String messageFmt, Object param1, Object param2) {
        this (messageFmt, null, new Object[]{param1,param2});
    }

    /**
     * Constructs and sets the <tt>message</tt> of this from a one
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     *
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 message parameter
     */
    public BootRuntimeException (String messageFmt, Object param1) {
        this (messageFmt, null, new Object[]{param1});
    }

    /**
     * Constructs and sets the message to the provided value.
     *
     * @param message message text.
     */
    public BootRuntimeException (String message) {
        this (message, null, null);
    }

    /**
     * Constructs and sets the <tt>message</tt> from a format string as
     * defined in <tt>java.text.MessageFormat</tt>. The required parameters
     * must be contained in the <tt>params</tt> array.
     *
     * @param message    format string as defined in <tt>java.text.MessageFormat</tt>
     *                    or plain text that is not a MessageFormat, depending on <tt>params</tt>
     * @param l          a <tt>java.util.Locale</tt> to format locale
     *                    dependent data types or <tt>null</tt> if the
     *                    <tt>params</tt> do not contain local specifics.
     * @param params the parameters of the message. If <tt>params</tt> is <tt>null</tt>,
     *                message is not subject to MessageFormat-processing. If <tt>params</tt>
     *                is not <tt>null</tt>, <tt>message</tt> must be compliant to
     *                <tt>java.text.MessageFormat</tt>.
     */
    public BootRuntimeException (String message, Locale l, Object[] params) {
        setMessage (message, l, params);
    }

    /**
     * Sets the <tt>message</tt> from a format string as
     * defined in <tt>java.text.MessageFormat</tt>. The required parameters
     * must be contained in the <tt>params</tt> array.
     *
     * @param message    format string as defined in <tt>java.text.MessageFormat</tt>
     *                    or plain text that is not a MessageFormat, depending on <tt>params</tt>
     * @param l          a <tt>java.util.Locale</tt> to format locale
     *                    dependent data types or <tt>null</tt> if the
     *                    <tt>params</tt> do not contain local specifics.
     * @param params the parameters of the message. If <tt>params</tt> is <tt>null</tt>,
     *                message is not subject to MessageFormat-processing. If <tt>params</tt>
     *                is not <tt>null</tt>, <tt>message</tt> must be compliant to
     *                <tt>java.text.MessageFormat</tt>.
     * @return this
     */
    public BootRuntimeException setMessage (String message, Locale l, Object[] params) {
        if (message == null) {
            message_ = "";
            return this;
        }
        if (params == null) {
            message_ = message;
        } else {
            try {
                MessageFormat       mf = new MessageFormat (message);
                if (l != null) mf.setLocale(l);
                message_ = mf.format (params);
            } catch (Exception ex) {
                message_ = ex.toString();
            }
        }
        return this;
    }

    /**
     * Sets the <tt>message</tt> of this from a one
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     *
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 message parameter
     * @param param2 message parameter
     * @return this
     */
    public BootRuntimeException setMessage (String messageFmt, Object param1, Object param2) {
        setMessage (messageFmt, null, new Object[]{param1, param2});
        return this;
    }

    /**
     * Sets the <tt>message</tt> of this from a one
     * parameter <tt>java.text.MessageFormat</tt> compatible string.
     *
     * @param messageFmt format string as defined in <tt>java.text.MessageFormat</tt>.
     * @param param1 message parameter
     * @return this
     */
    public BootRuntimeException setMessage (String messageFmt, Object param1) {
        setMessage (messageFmt, null, new Object[]{param1});
        return this;
    }

    /**
     * Constructs and sets the message to the provided value.
     *
     * @param message text to set
     * @return this
     */
    public BootRuntimeException setMessage (String message) {
        setMessage (message, null, null);
        return this;
    }

    /**
     * Sets the code. The code must be greater than zero.
     * @see #getCode()
     */
    public BootRuntimeException setCode (int code) {
        if (code <= 0) throw new IllegalArgumentException();
        code_ = code;
        return this;
    }

    /**
     * Returns the code set. The code should be used to discrimitate amongst
     * different kinds of notifications.
     */
    public int getCode () {
        return code_;
    }

    /**
     * @return String message
     */
    public String getMessage() {
        return message_ == null ? "" : message_;
    }

    /**
     * Sets the detail exception. This usually is the technical exception that
     * is the root cause of this exception.
     *
     * @param detail the root cause technical exception.
     */
    private void setDetail (Throwable detail) {
        initCause(detail);
        detail_ = detail;
    }

    /**
     * Returns the detail throwable. The returned value is <tt>null</tt> if
     * no <tt>detail</tt> has been set or this has been <tt>migrated</tt> and
     * the <tt>detail</tt> is not an <tt>BaseException</tt>.
     */
    public Throwable getDetail () {
        return detail_;
    }


    public BootRuntimeException setShowToEndUser( boolean bIn ){
        showToEndUser_ = bIn;
        return this;
    }

    public boolean getShowToEndUser( ){
        return showToEndUser_;
    }

}
